#!/usr/bin/perl -w
#
# Add listed files to binary
# Copyright (C) 2000, Kostromitinov Pavel
#
BEGIN {					# Hack to search for modules in current folder
	$0=~/^(.*)[\/\\:]/;
	push (@INC,$1) if (defined $1)&&($1 ne '');
}

use FNdxCommon;
use FNdxConfig;
use FNdxBinary;
use Data::Dumper;
BEGIN {do 'FNdxExport.pm';}

my @Files;
my @BinAreas;

ProcessCommandLine(\&ShowHelp);
if ($Config{'BinOutput'} eq '') {die "Binary file not defined.\n";}
if (@ARGV==0) {ShowHelp();exit(0);}

foreach my $UpdList (@ARGV) {LoadUpdList($UpdList);}

print "Loading binary file...\n" if $Config{'Verbose'};
$BINVersion=0;
($IsValidBin,undef)=LoadBinaryFile($Config{'BinOutput'},$CurrentVersion,\@BinAreas);

@Files=sort {$a->[0] cmp $b->[0]} @Files;
foreach my $UpdFile (@Files) {LoadFile($UpdFile);}

print "Saving binary file...\n" if $Config{'Verbose'};
SaveBinaryFile($Config{'BinOutput'},$CurrentVersion,\@BinAreas);

#--------------------------------------

sub LoadUpdList {
	my ($FileName)=@_;
	unless (open(UPDLIST,"<$FileName")) {print "Cannot open '$FileName': $!\n";return;}
	while (defined (my $Line=<UPDLIST>)) {
		$Line=~/^\s*(\S+)\s+((".+")|(\S+))/;
		push(@Files,[$1,(defined($3))?substr($3,1,length($3)-2):$4]);
	}
	close (UPDLIST);
}

sub LoadFile {
	my ($UpdFile)=@_;
	my ($Area,$BINArea,$BINFile,$Found);

#	Search for area in configuration
	$Found=0;
	foreach $Area (@{$Config{'Areas'}}) {
		if ($Area->{'Tag'} eq $UpdFile->[0]) {$Found=$Area;last;}
	}
	unless ($Found) {print STDERR "Area not found: '$UpdFile->[0]'\n";return;}
    $Area=$Found;

#Search for area already in binary
	$Found=0;
	foreach $BINArea (@BinAreas) {
		if ($BINArea->{'Tag'} eq $UpdFile->[0]) {$Area=$BINArea;$Found=1;last;}
	}
	push(@BinAreas,$Area) unless ($Found);

#Read FILES.BBS
	unless (exists $Area->{'BBS'}) {
		$Area->{'BBS'}=$Area->ReadFilesBBS();
	}
	my $FullName=$Area->{'Path'}.$UpdFile->[1];
	my @Stat=stat($FullName);
	unless (@Stat) {print STDERR "File not found: '$FullName'\n",return;}
	my $ThisFile=new FNdxFile($UpdFile->[1],[''],0,$Stat[7],$Stat[9],$Area);

#	Check for file already in binary
	$Found=0;
	foreach $BINFile (@{$Area->{'Files'}}) {
		if ($BINFile->{'Name'} eq $UpdFile->[1]) {$Found=$BINFile;last;}
	}

#	Check for file in FILES.BBS
	foreach my $BBSFile (@{$Area->{'BBS'}}) {
		print $BBSFile->{'Name'},"\n";
		if (uc($BBSFile->{'Name'}) eq uc($UpdFile->[1])) {
			$ThisFile->{'Description'}=$BBSFile->{'Description'};last;
		}
	}

	if ($Found) {
		$Found->{'Description'}=$ThisFile->{'Description'};
	} else {
		push(@{$Area->{'Files'}},$ThisFile);
	}
}

sub ShowHelp {
	print <<'EOH';
Binary file updater version 1.00beta for FileNdx 4.52
Usage: AddFiles.pl [options] UpdList UpdList...
	Options are the same as for FileNdx
	UpdList has the form:

	Areatag filename
	...
EOH
}
