# FileNdx 4.52 - Area class
# Copyright (C) 2000, Kostromitinov Pavel
#
package FNdxArea;
use Time::Local;
use PKConfig;
use PKFormat;
use FNdxCommon;
use FNdxFile;
require Exporter;
@ISA=qw(Exporter);
@EXPORT=qw(ReadFilesBBS);

BEGIN {do 'FNdxExport.pm';}

# Type can be:
# 0 - basic
# 1 - subarea
# 2 - offline
sub new {
	my ($Class,$Tag,$Path,$Access,$Description,$Number,$Type)=@_;
	$Class=ref $Class if ref $Class;
	my %Area=('Tag'=>$Tag,'Path'=>$Path,'AccessLevel'=>$Access,
		'Description'=>$Description,'Params'=>{},'Number'=>$Number,'Type'=>$Type);
	return bless \%Area,$Class;
}

sub LoadBin {
	my ($Class,$DataRef)=@_;
	my %Area=('Files'=>[],'Params'=>{});
	my ($Data,$Count);
	$Class=ref $Class if ref $Class;
	$Area{'Path'}=LoadString($DataRef);
	$Area{'Tag'}=LoadString($DataRef);
	$Area{'Description'}=LoadString($DataRef);
    if ($BINVersion>=442) {
		($Area{'AccessLevel'},$Area{'Type'},$Count)=unpack('VCV',$$DataRef);
		substr($$DataRef,0,9,'');
    } else {
		($Area{'AccessLevel'},$Count)=unpack('VV',$$DataRef);
        $Area{'Type'}=0;
		substr($$DataRef,0,8,'');
    }

	for (my $I=0;$I<$Count;$I++) {
		my $File=LoadBin FNdxFile($DataRef);
		push(@{$Area{'Files'}},$File);
	}
	return bless \%Area,$Class;
}

sub ReadFilesBBS {
	my ($This)=@_;
	my $FileName=$This->{'Path'}.$Config{'ListFile'};
	my @Listed;

	if (open(LISTFILE,$FileName)) {
		my $DescFile=undef;
		my $Pattern='^(\\s*\\'.$Config{'MLChar'}.'\\s*|\\s+)(.*)$';
		while (defined(my $Line=<LISTFILE>)) {
			chomp $Line;
			next if $Line=~/^\s*$/;
			if ($Line=~/$Pattern/) {
				push(@{$DescFile->{'Description'}},$2) if (defined $DescFile);
				next;
			}
			unless ($Line=~/^((".+")|(\S+))\s*(\[(\d+)\])?\s*(.*?)\s*$/) {
				die("Invalid line in '$FileName':\n$Line\n");
			}
			my $ListFileName=(defined($2))?substr($2,1,length($2)-2):$3;
			$DescFile=new FNdxFile($ListFileName,[$6],(defined $5)?0+$5:0,0,0,$This);
			push(@Listed,$DescFile);
		}
		close(LISTFILE);
		return \@Listed;
	} else {
		print STDERR "Warning: cannot open '$FileName': $!\n";
		return [];
	}
}

sub Rescan {
	my ($This,$Logged)=@_;
	my ($FileName,$NameOnly);
	croak('Invalid array ref passed') if ref($Logged) ne 'ARRAY';
	$This->{'Files'}=[];

	if ($This->{'Type'}==2) {$This->ScanOffline($Logged);return;}

	unless (opendir DIRHANDLE,$This->{'Path'}) {
		print STDERR "Cannot open directory '",$This->{'Path'},"'\n";
		return;
	}
	print $This->{'Path'} if $Config{'Verbose'};
	while ($FileName=readdir DIRHANDLE) {
		$FileName=$This->{'Path'}.$FileName;
		next unless -f $FileName;
		$NameOnly=(ParseFile($FileName))[0];
		next if IsFileMask($NameOnly,@{$Config{'Exclude'}});

		my @Stat=stat($FileName);
		my $File=new FNdxFile($NameOnly,[],0,$Stat[7],$Stat[9],$This);
		push(@{$This->{'Files'}},$File);
	}
	closedir DIRHANDLE;
#-------- Reading FILES.BBS ---------
	my $Listed=$This->ReadFilesBBS();
	print "\r",' 'x length($This->{'Path'}),"\r" if $Config{'Verbose'};
#-------- Setting descriptions ---------
	my $NeedSaveBBS=$Config{'SmartListUpdate'}?0:1;
	foreach my $File (@{$This->{'Files'}}) {
		my $UCName=uc($File->{'Name'});
		my $CounterAdded=0;

		foreach my $LogFile (@$Logged) {
			if (uc($LogFile->[1]) eq $UCName) {
				$File->{'Counter'}=$File->{'Counter'}+1;
				$NeedSaveBBS=1;$CounterAdded=1;
			}
		}
		foreach my $ListFile (@$Listed) {
			if ($UCName eq uc($ListFile->{'Name'})) {
				@{$File->{'Description'}}=@{$ListFile->{'Description'}};
				$ListFile->{'Counter'}=$ListFile->{'Counter'}+1 if $CounterAdded;
				$File->{'Counter'}=$ListFile->{'Counter'};
				$ListFile->{'Used'}=1;
				last;
			}
		}
		if (@{$File->{'Description'}}==0) {
			my $AddFile=new FNdxFile('',[],0,0,0,$This);
			$File->GetDescription($This->{'Path'});
			%$AddFile=%$File;
			$AddFile->{'Used'}=1;
			push(@Listed,$AddFile);
			$NeedSaveBBS=1;
		}
	}
	@{$This->{'Files'}}=sort {$a->{'Name'} cmp $b->{'Name'}} @{$This->{'Files'}};
	$This->SaveListFile(\@Listed) if ($NeedSaveBBS);
}

sub ScanOffline {
	my ($This,$Logged)=@_;
	my ($Pattern,$Line,$File);

    $Pattern='^(\\s*\\'.$Config{'MLChar'}.'\\s*|\\s+)(.*)$';
    open(OFFLINE,$This->{'Path'}) or die("Cannot open '".$This->{'Path'},"': $!\n");
	while (defined($Line=<OFFLINE>)) {
		chomp $Line;
		if ($Line=~/$Pattern/) {
			push(@{$File->{'Description'}},$2) if (defined $File);
			next;
		}
		unless ($Line=~/^((".+")|(\S+))\s+(\d+)\s+(\S+)\s+(\S+)\s+(\[(\d+)\])?\s*(.*?)\s*$/) {
			die("Invalid line in '".$This->{'Path'},"':\n$Line\n");
		}
		my ($FileName,$Size,$Date,$Time,$Counter,$Descr)=(
			(defined($2))?substr($2,1,length($2)-2):$3,
			$4,$5,$6,
            (defined $8)?0+$8:0,
            $9
		);
		my @DateTime;
		unless ($Date=~/^(\d{1,2})[.\/](\d{1,2})[.\/](\d{1,4})$/) {
			die("Invalid date in '".$This->{'Path'},"':\n$Date\n");
		}
		$DateTime[3]=$1;
		$DateTime[4]=$2-1;
		$DateTime[5]=($3<100)?$3:$3-1900;
		unless ($Time=~/^(\d{1,2}):(\d{1,2})(:(\d{1,2}))?$/) {
			die("Invalid time in '".$This->{'Path'},"':\n$Time\n");
		}
		$DateTime[0]=(defined $4)?$4:0;
		$DateTime[1]=$2;
		$DateTime[2]=$1;
		$File=new FNdxFile($FileName,[$Descr],$Counter,$Size,timelocal(@DateTime),$This);
		push(@{$This->{'Files'}},$File);
	}
	close(OFFLINE);
	@{$This->{'Files'}}=sort {$a->{'Name'} cmp $b->{'Name'}} @{$This->{'Files'}};
}

sub SaveListFile {
	my ($This,$Listed)=@_;
	my $FileName=$This->{'Path'}.$Config{'ListFile'};
	if (open(LISTFILE,">$FileName")) {
		foreach my $File (@$Listed) {
			$File->SaveListFile(*LISTFILE{IO}) if ((!$Config{'DeleteNonExisting'})||($File->{'Used'}));
		}
		close(LISTFILE);
	} else {
		print STDERR "Warning: cannot create '$FileName': $!\n";
	}
}

sub Count {
	my ($This)=@_;
	my $Count=0;
	foreach my $File (@{$This->{'Files'}}) {
		$Count++ if ($WhatSaving==0)&&($File->IsGood());
		$Count++ if ($WhatSaving==1)&&($File->IsNew());
	}
	return $Count;
}

sub FileSize {
	my ($This)=@_;
	my $FileSize=0;
	foreach my $File (@{$This->{'Files'}}) {
		$FileSize+=$File->{'Size'} if ($WhatSaving==0)&&($File->IsGood());
		$FileSize+=$File->{'Size'} if ($WhatSaving==1)&&($File->IsNew());
	}
	return $FileSize;
}

sub Calculate {
	my ($This)=@_;
	my $Total=$This->FileSize();

	$This->{'Params'}={
		'N'=>$This->{'Number'},
		'T'=>$This->{'Tag'},
		'D'=>$This->{'Description'},
		'S'=>$Total,
		'Z'=>Size($Total),
		'F'=>$This->Count(),
		'L'=>$This->{'AccessLevel'},
		'V'=>'Unknown',
		'W'=>$This->{'Path'},
		'P'=>ProcessPath($This->{'Path'})
	};
	foreach my $Acc (@{$Config{'AccessLevel'}}) {
		if ($Acc->{'Value'}>$This->{'AccessLevel'}) {last;} else {
			$AreaParams{'V'}=$Acc->{'Description'};
		}
	}
}

sub Prefix {
	my ($This)=@_;
	return ('','Sub','Offline')[$This->{'Type'}];
}

sub Save {
	my ($This)=@_;

	PrintUserTemplate(*OUTPUT{IO},$Templates{$This->Prefix().'AreaStart'},$This->{'Params'});
	if ($Config{'AddDirInfo'}&(1<<$WhatSaving)) {
		foreach my $DirInfo (@{$Config{'DirInfo'}}) {
			if (-f $This->{'Path'}.$DirInfo) {
				if (open(DIRINFO,$This->{'Path'}.$DirInfo)) {
					print OUTPUT <DIRINFO>;
					close(DIRINFO);
					print "\n";
					last;
				}
			}
		}
	}
	foreach my $File (@{$This->{'Files'}}) {
		next if (($WhatSaving==0)&&(!$File->IsGood()));
		next if (($WhatSaving==1)&&(!$File->IsNew()));
		$File->Save();
	}
	PrintUserTemplate(*OUTPUT{IO},$Templates{$This->Prefix().'AreaEnd'},$This->{'Params'});
}

sub SaveBin {
	my ($This,$FileRef)=@_;
    $FileRef=*OUTPUT{IO} unless defined $FileRef;
	SaveString($FileRef,$This->{'Path'},$This->{'Tag'},$This->{'Description'});
	print $FileRef pack('VCV',$This->{'AccessLevel'},$This->{'Type'},scalar @{$This->{'Files'}});
	foreach my $File (@{$This->{'Files'}}) {$File->SaveBin($FileRef);}
}

sub CheckBin {
	my ($This,$BinRef)=@_;
	foreach my $File (@{$This->{'Files'}}) {
		foreach my $BinFile (@{$BinRef->{'Files'}}) {
			if ($File->{'Name'} eq $BinFile->{'Name'}) {
				$File->{'Added'}=Max($BinFile->{'Added'},$File->{'Date'});
				last;
			}
		}
	}
}

sub Max {
	my $M=shift;
	foreach my $C (@_) {$M=$C if $C>$M;}
	return $M;
}

1;
