# FileNdx 4.52 - Configuration routines
# Copyright (C) 2000, Kostromitinov Pavel
#
package FNdxConfig;
require Exporter;
@ISA=qw(Exporter);
@EXPORT=qw(ProcessCommandLine LoadConfig AnalyzeString);

use Carp;
use FNdxCommon;
use FNdxArea;
use FNdxDescriptor;
use FNdxLogger;
use PKConfig;
use PKParams;
BEGIN {do 'FNdxExport.pm';}

sub ProcessCommandLine {
	my ($ShowHelp)=@_;
	$Path=(ParseFile($0))[1];
	$StringParams{'C'}=$Path."FileNdx.cfg";

	$Result=ParseCommandLine(\%SwitchParams,\%StringParams);
	die "Bad command-line parameter: $Result\n" if defined $Result;

#	if ($SwitchParams{'H'}) {ShowHelp();exit(0);};
	if ($SwitchParams{'H'}) {&$ShowHelp();exit(0);};

	foreach $String (@{$StringParams{'D'}}) {AnalyzeString('Set '.$String);}
	LoadConfig($StringParams{'C'});
	foreach $String (@{$StringParams{'I'}}) {LoadConfig($String);}
	foreach $String (@{$StringParams{'S'}}) {AnalyzeString($String);}

	@{$Config{'AccessLevel'}}=sort {($a->{'Value'})<=>($b->{'Value'})} @{$Config{'AccessLevel'}};
	if (@{$Config{'Descriptor'}}==0) {AddDescriptor($Config{'Descriptor'},'0 Public');}

	foreach $Key (keys %Templates) {LoadTemplate($Key);}
	$LastNewTime=$CurrentTime-$Config{'NewFileDays'}*86400;
	close(STDERR);
	die "Cannot open log file '",$Config{'Log'},"': $!\n" unless open(STDERR,'>'.$Config{'Log'});

	DumpConfiguration() if $SwitchParams{'G'}!=-1;
	exit 1 if $SwitchParams{'G'}==0;
	exit 1 if ($SwitchParams{'T'});
}

# LoadConfig(FileName)
sub LoadConfig {
	my ($FileName)=@_;
	local *CONFIG;
	open(CONFIG,$FileName) or die "Cannot open '$FileName':$!\n";
	while (defined (my $Line=ReadConfigLine(*CONFIG{IO}))) {
		AnalyzeString($Line);
	}
	close(CONFIG);
}

sub AnalyzeString {
	my ($String)=@_;
	my ($UString,$Value);

	($String,$Value,undef)=ParseConfigLine($String);
	return if $String eq '';
	die("Invalid configuration line: '$String'\n") unless defined $Value;
	$UString=uc($String);
	$Value=ProcessUserVars($Value);

#Booleans
	if ($UString eq 'SMARTLISTUPDATE') {$Config{'SmartListUpdate'}=Boolean($Value);return;};
	if ($UString eq 'DELETENONEXISTING') {$Config{'DeleteNonExisting'}=Boolean($Value);return;};
	if ($UString eq 'INCLUDEEMPTY') {$Config{'IncludeEmpty'}=Boolean($Value);return;};
	if ($UString eq 'USEMULTILINECHAR') {$Config{'UseMLChar'}=Boolean($Value);return;};
	if ($UString eq 'LOGANALYZER') {$Config{'LogAnalyzer'}=Boolean($Value);return;};
	if ($UString eq 'ADDCOPYRIGHT') {$Config{'AddCopyright'}=Boolean($Value);return;};
	if ($UString eq 'VERBOSE') {$Config{'Verbose'}=Boolean($Value);return;};
	if ($UString eq 'MARKNEWFILES') {$Config{'MarkNewFiles'}=Boolean($Value);return;};
#Numbers
	if ($UString eq 'LDESCSTART') {$Config{'LDescStart'}=Number($Value);return;};
	if ($UString eq 'WAITINPUT') {$Config{'WaitInput'}=Number($Value);return;};
	if ($UString eq 'DESCWIDTH') {$Config{'DescWidth'}->[0]=Number($Value);return;};
	if ($UString eq 'NEWDESCWIDTH') {$Config{'DescWidth'}->[1]=Number($Value);return;};
	if ($UString eq 'BESTDESCWIDTH') {$Config{'DescWidth'}->[2]=Number($Value);return;};
	if ($UString eq 'LISTFILECOUNTER') {$Config{'ListFileCounter'}=Number($Value);return;};
	if ($UString eq 'NEWFILEDAYS') {$Config{'NewFileDays'}=Number($Value);return;};
	if ($UString eq 'BESTFILES') {$Config{'BestFiles'}=Number($Value);return;};
	if ($UString eq 'DEFAULTACCESSLEVEL') {$Config{'DefaultAccessLevel'}=Number($Value);return;};
	if ($UString eq 'SIZELIMIT1') {$Config{'Limit1'}=Number($Value);return;};
	if ($UString eq 'SIZELIMIT2') {$Config{'Limit2'}=Number($Value);return;};
	if ($UString eq 'MAXSFNLENGTH') {$Config{'MaxSFNLength'}=Number($Value);return;};
#Existing Files
	if ($UString eq 'HEADER') {$Config{'Header'}=FileExists($Value);return;};
	if ($UString eq 'FOOTER') {$Config{'Footer'}=FileExists($Value);return;};
	if ($UString eq 'NEWHEADER') {$Config{'NewHeader'}=FileExists($Value);return;};
	if ($UString eq 'NEWFOOTER') {$Config{'NewFooter'}=FileExists($Value);return;};
	if ($UString eq 'BESTHEADER') {$Config{'BestHeader'}=FileExists($Value);return;};
	if ($UString eq 'BESTFOOTER') {$Config{'BestFooter'}=FileExists($Value);return;};
	if ($UString eq 'AREASTART') {$Config{'AreaStart'}=FileExists($Value);return;};
	if ($UString eq 'SUBAREASTART') {$Config{'SubAreaStart'}=FileExists($Value);return;};
	if ($UString eq 'OFFLINEAREASTART') {$Config{'OfflineAreaStart'}=FileExists($Value);return;};
	if ($UString eq 'AREAEND') {$Config{'AreaEnd'}=FileExists($Value);return;};
	if ($UString eq 'SUBAREAEND') {$Config{'SubAreaEnd'}=FileExists($Value);return;};
	if ($UString eq 'OFFLINEAREAEND') {$Config{'OfflineAreaEnd'}=FileExists($Value);return;};
	if ($UString eq 'AREALISTSTART') {$Config{'AreaListStart'}=FileExists($Value);return;};
	if ($UString eq 'AREALISTEND') {$Config{'AreaListEnd'}=FileExists($Value);return;};
	if ($UString eq 'NEWAREALISTSTART') {$Config{'NewAreaListStart'}=FileExists($Value);return;};
	if ($UString eq 'NEWAREALISTEND') {$Config{'NewAreaListEnd'}=FileExists($Value);return;};
#New files
	if ($UString eq 'OUTPUT') {$Config{'Output'}=FileNew($Value);return;};
	if ($UString eq 'NEWOUTPUT') {$Config{'NewOutput'}=FileNew($Value);return;};
	if ($UString eq 'BESTOUTPUT') {$Config{'BestOutput'}=FileNew($Value);return;};
	if ($UString eq 'BINOUTPUT') {$Config{'BinOutput'}=FileNew($Value);return;};
	if ($UString eq 'LOGBINARY') {$Config{'LogBinary'}=FileNew($Value);return;};
	if ($UString eq 'AREAFILE') {$Config{'AreaFile'}=StripQuotes($Value);return;};
	if ($UString eq 'NEWAREAFILE') {$Config{'NewAreaFile'}=StripQuotes($Value);return;};
	if ($UString eq 'LOG') {$Config{'Log'}=FileNew($Value);return;};
#Set, conditionals
	if ($UString eq 'SET') {
		my ($Var,$Val)=SetLine($Value);
		$UserVars{$Var}=$Val;
		return;
	};
	if ($UString eq 'DEFSET') {
		my ($Var,$Val)=SetLine($Value);
		$UserVars{$Var}=$Val unless defined $UserVars{$Var};
		return;
	};
	if (($UString eq 'UNSET')||($UString eq 'UNDEF')) {
		die("Invalid 'UnSet' line: '$Value'\n") unless $Value=~/^\s*(\w+)\s*$/;
		delete $UserVars{$1} if exists $UserVars{$1};
		return;
	};
	if ($UString eq 'IFDEF') {
		my ($Var,$Val)=SetLine($Value);
		AnalyzeString($Val) if defined $UserVars{$Var};
		return;
	};
	if ($UString eq 'IFNDEF') {
		my ($Var,$Val)=SetLine($Value);
		AnalyzeString($Val) unless defined $UserVars{$Var};
		return;
	};
	if ($UString eq 'IF') {
		my ($Condition,$Command)=IfLine($Value);
		AnalyzeString($Command) if eval($Condition);
		return;
	};
	if ($UString eq 'UNLESS') {
		my ($Condition,$Command)=IfLine($Value);
		AnalyzeString($Command) unless eval($Condition);
		return;
	};
#Strings
	if ($UString eq 'LISTFILE') {$Config{'ListFile'}=StripQuotes($Value);return;};
	if ($UString eq 'NEWMARK') {$Config{'NewMark'}=Word($Value);return;};
	if ($UString eq 'NOTNEWMARK') {$Config{'NotNewMark'}=Word($Value);return;};
	if ($UString eq 'SIZETAG1') {$Config{'Tag1'}=Word($Value);return;};
	if ($UString eq 'SIZETAG2') {$Config{'Tag2'}=Word($Value);return;};
	if ($UString eq 'SIZETAG3') {$Config{'Tag3'}=Word($Value);return;};
	if ($UString eq 'DEFAULTDESCRIPTION') {$Config{'DefaultDescription'}=StripQuotes($Value);return;};
	if ($UString eq 'EXCLUDEDESCRIPTION') {$Config{'ExcludeDescription'}=StripQuotes($Value);return;};
#Formats
	if ($UString eq 'FILEFORMAT') {$Config{'FileFormats'}->[0]=StripQuotes($Value);return;};
	if ($UString eq 'FILEFORMATCONT') {$Config{'FileFormatsCont'}->[0]=StripQuotes($Value);return;};
	if ($UString eq 'LFNFILEFORMAT') {$Config{'LFNFileFormats'}->[0]=StripQuotes($Value);return;};
	if ($UString eq 'LFNFILEFORMATCONT') {$Config{'LFNFileFormatsCont'}->[0]=StripQuotes($Value);return;};
	if ($UString eq 'NEWFILEFORMAT') {$Config{'FileFormats'}->[1]=StripQuotes($Value);return;};
	if ($UString eq 'NEWFILEFORMATCONT') {$Config{'FileFormatsCont'}->[1]=StripQuotes($Value);return;};
	if ($UString eq 'NEWLFNFILEFORMAT') {$Config{'LFNFileFormats'}->[1]=StripQuotes($Value);return;};
	if ($UString eq 'NEWLFNFILEFORMATCONT') {$Config{'LFNFileFormatsCont'}->[1]=StripQuotes($Value);return;};
	if ($UString eq 'BESTFILEFORMAT') {$Config{'FileFormats'}->[2]=StripQuotes($Value);return;};
	if ($UString eq 'BESTFILEFORMATCONT') {$Config{'FileFormatsCont'}->[2]=StripQuotes($Value);return;};
	if ($UString eq 'BESTLFNFILEFORMAT') {$Config{'LFNFileFormats'}->[2]=StripQuotes($Value);return;};
	if ($UString eq 'BESTLFNFILEFORMATCONT') {$Config{'LFNFileFormatsCont'}->[2]=StripQuotes($Value);return;};

	if ($UString eq 'AREALISTITEM') {$Config{'AreaListItems'}->[0]=StripQuotes($Value);return;};
	if ($UString eq 'SUBAREALISTITEM') {$Config{'AreaListItems'}->[1]=StripQuotes($Value);return;};
	if ($UString eq 'OFFLINEAREALISTITEM') {$Config{'AreaListItems'}->[2]=StripQuotes($Value);return;};
	if ($UString eq 'NEWAREALISTITEM') {$Config{'NewAreaListItems'}->[0]=StripQuotes($Value);return;};
	if ($UString eq 'NEWSUBAREALISTITEM') {$Config{'NewAreaListItems'}->[1]=StripQuotes($Value);return;};
	if ($UString eq 'NEWOFFLINEAREALISTITEM') {$Config{'NewAreaListItems'}->[2]=StripQuotes($Value);return;};

	if ($UString eq 'DATEFORMAT') {$Config{'DateFormat'}=StripQuotes($Value);return;};
#Other
	if ($UString eq 'MULTILINECHAR') {
		if (length($Value)==1) {$Config{'MLChar'}=StripQuotes($Value);return;} else {die("'$Var' is not a single character.\n");}
	};
	if ($UString eq 'ADDDIRINFO') {
		$Value=uc($Value);
		$Config{'AddDirInfo'}=0x00;
		if (($Value eq 'ALL')||($Value eq 'YES')) {$Config{'AddDirInfo'}=0x07;return;};
		if ($Value=~/\bNORMAL\b/) {$Config{'AddDirInfo'}|=0x01;};
		if ($Value=~/\bNEW\b/) {$Config{'AddDirInfo'}|=0x02;};
		if ($Value=~/\bBEST\b/) {$Config{'AddDirInfo'}|=0x04;};
		return;
	};
	if ($UString eq 'INCLUDEAREALIST') {
		$Value=NoNormalNewAll($Value);
		if (defined $Value) {$Config{'IncludeAreaList'}=StripQuotes($Value);return;};
		die("Invalid 'IncludeAreaList' value: '$Value'\n");
	};
	if ($UString eq 'MULTIPLEFILE') {
		$Value=NoNormalNewAll($Value);
		if (defined $Value) {$Config{'MultipleFile'}=StripQuotes($Value);return;};
		die("Invalid 'MultipleFile' value: '$Value'\n");
	};

	if ($UString eq 'PROMPTFORNEW') {
		$Value=uc($Value);$Bool=Boolean($Value,1);
		if (defined $Bool) {$Config{'PromptForNew'}=$Bool;return;};
		if ($Value eq 'AUTO') {$Config{'PromptForNew'}=2;return;};
		if ($Value eq 'MANUAL') {$Config{'PromptForNew'}=3;return;};
		die("Invalid 'PromptForNew' value: '$Value'\n");
	};
	if ($UString eq 'WRAPLONGDESCRIPTION') {
		$Value=uc($Value);$Bool=Boolean($Value,1);
		if (defined $Bool) {$Config{'WrapLong'}=$Bool;return;};
		if ($Value eq 'LINE') {$Config{'WrapLong'}=2;return;};
		die("Invalid 'WrapLong' value: '$Value'\n");
	};
	if ($UString eq 'SFNCASE') {$Config{'SFNCase'}=CaseConvert($Value);return;};
	if ($UString eq 'LFNCASE') {$Config{'LFNCase'}=CaseConvert($Value);return;};
   	if ($UString eq 'VERSIONSTAMP') {
   		if ($Value=~/^(\d+)\.(\d+)$/) {
			die("This is a FileNdx/DOS configuration file, not suppported\n") if $1<4;
			$Config{'VersionStamp'}=$1*100+$2;
			die("This configuration file is for a later version of the script\n")
				if $Config{'VersionStamp'}>$CurrentVersion;
   			return;
   		} else {
   			die("Invalid version stamp: '$Value'\n");
		}
   	}
#Lists
	if ($UString eq 'PROCESSPATH') {push(@{$Config{'ProcessPath'}},$Value);return;};
	if ($UString eq 'EXCLUDE') {AddWords($Config{'Exclude'},$Value);return;};
	if ($UString eq 'EXCLUDEFROMNEW') {AddWords($Config{'ExcludeFromNew'},$Value);return;};
	if ($UString eq 'EXCLUDEFROMBEST') {AddWords($Config{'ExcludeFromBest'},$Value);return;};
	if ($UString eq 'DIRINFO') {AddWords($Config{'DirInfo'},$Value);return;};
	if ($UString eq 'ANALYZELOG') {AddWords($Config{'Logs'},$Value);return;};
	if ($UString eq 'LOGSTRING') {AddLogger($Config{'LogStrings'},$Value);return;};
	if ($UString eq 'DESCRIPTOR') {AddDescriptor($Config{'Descriptor'},$Value);return;};
	if ($UString eq 'ACCESSLEVEL') {AddAccessLevel($Config{'AccessLevel'},$Value);return;};
	if ($UString eq 'AREA') {AddArea($Config{'Areas'},$Value,0);return;};
	if ($UString eq 'SUBAREA') {AddArea($Config{'Areas'},$Value,1);return;};
	if ($UString eq 'OFFLINEAREA') {AddArea($Config{'Areas'},$Value,2);return;};

	if ($UString eq 'INCLUDE') {LoadConfig($Value);return;};
	die("Unrecognized keyword: '$String'\n");
}

sub CaseConvert {
	my $Value=uc(shift);
	if ($Value eq 'NONE') {return 0;}
	if ($Value eq 'PRESERVE') {return 0;}
	if ($Value eq 'LOWER') {return 1;}
	if ($Value eq 'UPPER') {return 2;}
	if ($Value eq 'CAPITAL') {return 3;}
	die("Invalid capitalization: '$Value'\n");
}

sub NoNormalNewAll {
	my $Value=uc(shift);
	return 0x00 if (($Value eq 'NO')||($Value eq 'NONE'));
	return 0x01 if ($Value eq 'NORMAL');
	return 0x02 if ($Value eq 'NEW');
	return 0x03 if (($Value eq 'ALL')||($Value eq 'YES'));
	return undef;
}

sub SetLine {
	my $Value=shift;
	die("Invalid 'Set' line: '$Value'\n") unless $Value=~/^(\w+)((\s+|\s*=\s*)(.*?)\s?)?$/;
	return ($1,defined($4)?StripQuotes($4):'');
}

sub IfLine {
	my $Value=shift;
	die("Invalid 'If' String: '$Value'\n") unless $Value=~/(\"(.+)\"|\S+)\s*(.*)/;
	return (defined($2)?$2:$1,$3);
}

sub AddWords {
	my ($ArrRef,$String)=@_;
	croak("Invalid array ref passed") if (ref($ArrRef) ne 'ARRAY');
	until ($String=~/^\s*$/) {
		if ($String=~/^\s*"(.*?)"\s*(.*)/) {push(@$ArrRef,$1);$String=$2;next;}
		$String=~/^\s*(\S+)\s*(.*)/;push(@$ArrRef,$1);$String=$2;
	};
}

sub AddLogger {
	my ($ArrRef,$String)=@_;
	croak("Invalid array ref passed") if (ref($ArrRef) ne 'ARRAY');
	croak("Invalid Log string: '$String'") unless $String=~/^(\d+)\s+(\d+)\s+(.+)\s*$/;
	push(@$ArrRef,new FNdxLogger($1,$2,$3));
}

sub AddDescriptor {
	my ($ArrRef,$String)=@_;
	croak("Invalid array ref passed") if (ref($ArrRef) ne 'ARRAY');
	croak("Invalid Descriptor string: '$String'") unless $String=~/^((.+?)\s*=)?\s*(.+?)(\s*,\s*(.+)\s*)?$/;
	my @Masks;
	if (defined $2) {AddWords(\@Masks,$2);} else {@Masks=('*');}
	push(@$ArrRef,new FNdxDescriptor(\@Masks,$3,$5));
}

sub AddAccessLevel {
	my ($ArrRef,$String)=@_;
	croak("Invalid array ref passed") if (ref($ArrRef) ne 'ARRAY');
	croak("Invalid AccessLevel string: '$String'") unless $String=~/^(\d+)\s+(.+?)\s*$/;
	$ALRef={'Value'=>$1,'Description'=>$2};
	push(@$ArrRef,$ALRef);
}

sub AddArea {
	my ($ArrRef,$String,$Type)=@_;
	my ($Tag,$Path,$Access,$Description);
	croak("Invalid array ref passed") if (ref($ArrRef) ne 'ARRAY');
	croak("Invalid Area string: '$String'") unless
		$String=~/^(\S+)\s+((".+")|(\S+?))(,\d+)?\s+(.*?)\s*$/;
	$Tag=$1;
	foreach my $Area (@$ArrRef) {
		die("Duplicate area tag: '$Tag'.\n") if $Area->{'Tag'} eq $Tag;
	}
	$Path=(defined($3))?substr($3,1,length($3)-2):$4;
	$Access=(defined($5))?substr($5,1):$Config{'DefaultAccessLevel'};
	$Description=$6;
	push(@$ArrRef,new FNdxArea($Tag,$Path,$Access,$Description,1+scalar @$ArrRef,$Type));
}

sub ShowHelp {
	print <<'EOH';
File Index Builder version 4.52/perl
Usage: FileNdx.pl [options]
Options:
	-Cfile        - Read configuration from 'file' (FileNdx.cfg by default)
	-Ifile        - Include 'file' to configuration
	-Sstring      - Add 'string' to configuration
	-DVar[=Value] - Define 'Var' with 'Value' or empty string
	-G[+|-]       - Dump configuration (with '-' - quit after)
	-T            - Only read configuration; do no real processing
	-V            - Verbose log (for debugging)
	-H            - This help
EOH
}

my $Indent=0;

sub PrintHash {
	my $Value=shift;
	$Indent++;print STDERR "\n";
	foreach $Key (sort keys(%$Value)) {
		print STDERR "\t"x$Indent,"$Key => ";
		PrintValue($Value->{$Key});
	}
	$Indent--;
}

sub PrintArray {
	my $Value=shift;
	$Indent++;print STDERR "\n";
	for (my $I=0;$I<scalar @$Value;$I++) {
		print STDERR "\t"x$Indent,"$I: ";
		PrintValue($Value->[$I]);
	}
	$Indent--;
}

sub PrintValue {
	foreach my $Value (@_) {
		if (ref $Value eq 'ARRAY') {PrintArray($Value);next;}
		if (ref $Value eq 'HASH') {PrintHash($Value);next;}
		if (ref $Value eq 'SCALAR') {PrintValue($$Value);next;}
		unless (ref $Value) {print STDERR "'$Value'\n";next;}
        PrintHash($Value);
	}
}

sub DumpConfiguration {
	print STDERR "Configuration dump:\n";
	PrintHash(\%Config);
	print STDERR "\n\nUser Variables:\n";
	PrintHash(\%UserVars);
}

1;
