# File class
# Copyright (C) 2000, Kostromitinov Pavel
#
package FNdxFile;
use Carp;
use PKFormat;
use FNdxCommon;
use FNdxConfig;
use Data::Dumper;
BEGIN {do 'FNdxExport.pm';}

sub new {
	my ($Class,$Name,$Description,$Counter,$Size,$Date,$Area)=@_;
	croak('Invalid array ref passed') unless ref $Description eq 'ARRAY';
	my %File=('Name'=>$Name,'Description'=>$Description,'Counter'=>$Counter,
		'Size'=>$Size,'Date'=>$Date,'Added'=>($IsValidBin)?$CurrentTime:$Date,'Area'=>$Area);
	$Class=ref $Class if ref $Class;
	return bless \%File,$Class;
}

sub LoadBin {
	my ($Class,$DataRef)=@_;
	my %File=('Description'=>[]);

    my ($Data,$Count);
	$Class=ref $Class if ref $Class;
	$File{'Name'}=LoadString($DataRef);

    if ($BINVersion>=441) {
		($File{'Size'},$File{'Date'})=unpack('VV',$$DataRef);
		substr($$DataRef,0,8,'');
	} else {
	    $File{'Size'}=0;
	    $File{'Date'}=time;
	}

	($File{'Added'},$File{'Counter'},$Count)=unpack('VVV',$$DataRef);
	substr($$DataRef,0,12,'');

	for (my $I=0;$I<$Count;$I++) {
		my $Line=LoadString($DataRef);
		push(@{$File{'Description'}},$Line);
	}
	return bless \%File,$Class;
}

sub IsGood {
	$This=shift;
	return 0 if ((scalar @{$This->{'Description'}}==1)&&($This->{'Description'}->[0] eq $Config{'ExcludeDescription'}));
	return 1;
}

sub IsNew {
	$This=shift;
	return ($This->{'Added'}>$LastNewTime)?$This->IsGood():0;
}

sub Save {
	my ($This)=@_;
	my $Wrap=WrapDescription($This->{'Description'});
	my %FileParams=(
		'A'=>$This->{'Area'}->{'Number'},
		'G'=>$This->{'Area'}->{'Tag'},
		'N'=>ChangeCase($This->{'Name'}),
		'S'=>$This->{'Size'},
		'Z'=>Size($This->{'Size'}),
		'T'=>Date($This->{'Date'}),
		'D'=>$Wrap->[0],
		'C'=>$This->{'Counter'},
		'I'=>($Config{'MarkNewFiles'}&&$This->IsNew())?$Config{'NewMark'}:$Config{'NotNewMark'},
		'W'=>$This->{'Area'}->{'Path'},
		'P'=>ProcessPath($This->{'Area'}->{'Path'})
	);
	my $Format=(length($This->{'Name'})>12)?'LFNFileFormats':'FileFormats';
	print OUTPUT ExpandUserString($Config{$Format}->[$WhatSaving],\%FileParams),"\n";
	shift @$Wrap;$Format.='Cont';
	while (@$Wrap>0) {
		$FileParams{'D'}=shift @$Wrap;
		print OUTPUT ExpandUserString($Config{$Format}->[$WhatSaving],\%FileParams),"\n";
	}
}

sub ChangeCase {
	my ($FileName)=@_;
	my $How=(length($FileName)>12)?$Config{'LFNCase'}:$Config{'SFNCase'};
	return lc($FileName) if ($How==1) ;
	return uc($FileName) if ($How==2) ;
	return ucfirst(lc($FileName)) if ($How==3);
	return $FileName;
}

sub WrapDescription {
	my $Desc=shift;
	my ($WrapBy,$IsWrap)=($Config{'DescWidth'}->[$WhatSaving],$Config{'WrapLong'});
	my ($All,$Wrap,$Line);
	croak('Invalid array ref passed') if ref($Desc) ne 'ARRAY';
	if (($IsWrap==0)||(($IsWrap==2)&&(@$Desc>1))) {
		@$Wrap=@$Desc;
		push (@$Wrap,'') if (@$Desc==0);
		return $Wrap;
	}
	$All=join(' ',@$Desc);$Wrap=[];$Line='';
	until ($All=~/^\s*$/) {
		$All=~/^\s*(\S+)\s*(.*)$/;
		if ((length($Line.' '.$1)>$WrapBy)&&($Line ne '')) {
			push(@$Wrap,$Line);$Line=$1;
		} else {
			$Line.=' ' if $Line ne '';
			$Line.=$1;
		}
		$All=$2;
	}
	push (@$Wrap,$Line) if $Line ne '';
	return $Wrap;
}

sub SaveBin {
	my ($This,$FileRef)=@_;
    $FileRef=*OUTPUT{IO} unless defined $FileRef;
	SaveString($FileRef,$This->{'Name'});
	print $FileRef pack('VVVVV',$This->{'Size'},$This->{'Date'},$This->{'Added'},$This->{'Counter'},scalar @{$This->{'Description'}});
	foreach my $Line (@{$This->{'Description'}}) {SaveString($FileRef,$Line);}
}

sub GetDescription {
	my ($This,$Path,$Prompt)=(shift,shift,$Config{'PromptForNew'});
	print STDERR "New file found: $Path",$This->{'Name'},"\n";
	if (($Prompt==1)||($Prompt==2)) {
		foreach my $Descr (@{$Config{'Descriptor'}}) {
			if ($Descr->Applies($This->{'Name'})) {
				my $Result=$Descr->GetDescription($Path,$This->{'Name'});
				if (defined $Result) {
					$This->{'Description'}=$Result;
					print STDERR "Successfully auto-described\n";
					return;
				}
			}
		}
	}
	if (($Prompt==1)||($Prompt==3)) {
		my $Line;
		print 'Enter description for ',$This->{'Name'},":\n";
		while (my $Line=<STDIN>) {
			chomp $Line;
			push(@{$This->{'Description'}},$Line);
		}
		print STDERR "Successfully manually described\n";
		return;
	}
	$File->{'Description'}=[$Config{'DefaultDescription'}];
}

sub SaveListFile {
	my ($This,$FileRef)=@_;
	my ($Name,$Counter)=(ChangeCase($This->{'Name'}),$Config{'ListFileCounter'});

	$Name='"'.$Name.'"' if $Name=~/\s/;
	print {$FileRef} $Name,' 'x($Config{'LDescStart'}-length($Name)-2),' ';
	if ($Config{'ListFileCounter'}>0) {
		printf {$FileRef} "[%0*u] ",$Counter,$This->{'Counter'};
	}
	print {$FileRef} $This->{'Description'}->[0],"\n";
	for (my $I=1;$I<scalar @{$This->{'Description'}};$I++) {
		my $Shift=$Config{'LDescStart'}+(($Counter>0)?$Counter+2:-1);
		if ($Config{'UseMLChar'}) {
			print {$FileRef} $Config{'MLChar'},$This->{'Description'}->[$I],"\n";
		} else {
			print {$FileRef} ' 'x$Shift,$This->{'Description'}->[$I],"\n";
		}
	}
}

1;
