#!/usr/bin/perl -w
#
# FileNdx 4.52 - file indexing program
# Copyright (C) 2000, Kostromitinov Pavel
#
BEGIN {
	$0=~/^(.*)[\/\\:]/;			# Search for modules in current folder
	push (@INC,$1) if (defined $1)&&($1 ne '');
	$StartTime=time();
}

use PKFormat;
use FNdxCommon;
use FNdxLogger;
use FNdxConfig;
use FNdxBinary;
BEGIN {do 'FNdxExport.pm';}

ProcessCommandLine(\&FNdxConfig::ShowHelp);
print STDERR "FileNdx.pl started at ",scalar localtime,"\n";

print Elapsed(),"Started.\n" if $Config{'Verbose'};
#------------------------------------------------------
my %GlobalParams;
my @BinAreas=();
my ($Exclude,$Size,$Files,$Areas,$I,$BINCreated);

if ($Config{'BinOutput'} ne '') {
	print Elapsed(),"Loading binary file...\n" if $Config{'Verbose'};
	$BINVersion=0;
	($IsValidBin,$BINCreated)=LoadBinaryFile($Config{'BinOutput'},$CurrentVersion,\@BinAreas);
}

my $Logged;
print Elapsed(),"Analyzing log files...\n" if $Config{'Verbose'};
if ($Config{'LogAnalyzer'}) {$Logged=FNdxLogger::AnalyzeLogs()} else {$Logged=[];}

print Elapsed(),"Scanning areas...\n" if $Config{'Verbose'};
foreach my $Area (@{$Config{'Areas'}}) {
	$Area->Rescan($Logged);
	if ($IsValidBin) {
		foreach my $BinArea (@BinAreas) {
			if ($Area->{'Tag'} eq $BinArea->{'Tag'}) {
				$Area->CheckBin($BinArea);
				last;
			}
		}
	}
}

if ($Config{'Output'} ne '') {
	print Elapsed(),"Saving filelist(s)...\n" if $Config{'Verbose'};
	SaveAll(0);
}
if ($Config{'NewOutput'} ne '') {
	print Elapsed(),"Saving new files list(s)...\n" if $Config{'Verbose'};
	SaveAll(1);
}

SaveBest() if ($Config{'BestOutput'} ne '');

if ($Config{'BinOutput'} ne '') {
	print Elapsed(),"Saving binary file...\n" if $Config{'Verbose'};
	SaveBinaryFile($Config{'BinOutput'},$CurrentVersion,$Config{'Areas'});
}

print Elapsed(),"Finished.\n" if $Config{'Verbose'};
print STDERR "FileNdx.pl finished at ",scalar localtime,"\n";
close(STDERR);
#---------------------------------------------------------------------
sub SaveAll {
	$WhatSaving=shift;
	SavePrepare();

	if ($Config{'MultipleFile'}&&(1<<$WhatSaving)) {
		SaveAllMultiple();
	} else {
		SaveAllSingle();
	}
}

sub SaveAllSingle {
	SaveStart();
	foreach my $Area (@SaveAreas) {$Area->Save();}
	SaveAreaList() if ($Config{'IncludeAreaList'}&(1<<$WhatSaving));
	SaveFinish();
}

sub SaveAllMultiple {
	SaveStart();
	SaveAreaList();
	SaveFinish();
	foreach my $Area (@SaveAreas) {
		my $AreaFile=$Config{WhatSavingPrefix().'AreaFile'};
		$AreaFile=ExpandStringNoCR(ProcessUservars($AreaFile),$Area->{'Params'});
		CreateDirForFile($AreaFile);
		open(OUTPUT,">$AreaFile") or die("Cannot open output file '$AreaFile': $!\n");
		$Area->Save();
		close(OUTPUT);
	}
}

sub SavePrepare {
	($Size,$Files,$Areas)=(0,0,0,0);
	$Exclude=([],$Config{'ExcludeFromNew'})[$WhatSaving];

	@SaveAreas=();
	foreach my $Area (@{$Config{'Areas'}}) {
		$Area->Calculate();
		next if (IsFileMask($Area->{'Tag'},@$Exclude));
		my $Count=$Area->{'Params'}->{'F'};
		if ($Config{'IncludeEmpty'}||$Count) {
			push(@SaveAreas,$Area);
			$Size+=$Area->{'Params'}->{'S'};
			$Files+=$Count;$Areas++;
		}
	}
}

sub SaveStart {
	%GlobalParams=(
		'S'=>$Size,
		'Z'=>Size($Size),
		'F'=>$Files,
		'N'=>$Areas
	);
	open(OUTPUT,'>'.$Config{WhatSavingPrefix().'Output'}) or die("Cannot open output file: $!\n");
	PrintUserTemplateFromFile(*OUTPUT{IO},$Config{WhatSavingPrefix().'Header'},\%GlobalParams);
}

sub SaveAreaList {
	PrintUserTemplateFromFile(*OUTPUT{IO},$Config{WhatSavingPrefix().'AreaListStart'},\%GlobalParams);
	foreach $Area (@SaveAreas) {
		print OUTPUT ExpandUserString($Config{WhatSavingPrefix().'AreaListItems'}->[$Area->{'Type'}],$Area->{'Params'}),"\n";
	}
	PrintUserTemplateFromFile(*OUTPUT{IO},$Config{WhatSavingPrefix().'AreaListEnd'},\%GlobalParams);
}

sub SaveFinish {
	PrintUserTemplateFromFile(*OUTPUT{IO},$Config{WhatSavingPrefix().'Footer'},\%GlobalParams);
	PrintCopyright();
	close(OUTPUT);
}

sub SaveBest {
	print Elapsed(),"Looking for best files...\n" if $Config{'Verbose'};
	$WhatSaving=2;
	my @BestFiles=();
	foreach my $Area (@{$Config{'Areas'}}) {
		next if (IsFileMask($Area->{'Tag'},@{$Config{'ExcludeFromBest'}}));
		foreach my $File (@{$Area->{'Files'}}) {
			my $C=$File->{'Counter'};
			next if $C==0;
			next unless $File->IsGood();
			for ($I=0;$I<@BestFiles;$I++) {
				last if ($C>$BestFiles[$I]->{'Counter'});
			}
			splice(@BestFiles,$I,0,$File);
			splice(@BestFiles,$Config{'BestFiles'});
		}
	}

	($Size,$Files,$Areas)=(0,0,scalar @{$Config{'Areas'}});
	for my $File (@BestFiles) {
		$Files++;
		$Size+=$File->{'Size'};
	}

	print Elapsed(),"Saving best files list...\n" if $Config{'Verbose'};
	SaveStart();
	for my $File (@BestFiles) {$File->Save();}
	SaveFinish();
}

sub PrintCopyright {
	if ($Config{'AddCopyright'}) {
		print OUTPUT "\n";
		print OUTPUT "ͻ\n";
		print OUTPUT " Generated by FileNdx 4.52/perl                 \n";
		print OUTPUT " Copyright (C) 1998-2000 by Pavel Kostromitinov \n";
		print OUTPUT "Ķ\n";
		print OUTPUT " AKA 2:5030/661.14    E-Mail: kostrom\@mail.ru   \n";
		print OUTPUT "ͼ\n";
	}
}

sub Elapsed {
	my $Time=time()-$StartTime;
	return sprintf("%02d:%02d ",$Time/60,$Time%60);
}
