# FreqMgr 2.10 - Common varables and routines
# Copyright (C) 2.10, Kostromitinov Pavel
#
package FMgrCommon;
require Exporter;
use FMgrLimit;
@ISA=qw(Exporter);
@EXPORT=qw(Log LogDie DebugLog ParseAccess CalculateLimits AddGroupAndLevel AddPassword);

%SwitchParams=(
	'H'=>0
	);
%StringParams=(
	'C'=>'',
	'S'=>[],
	'I'=>[],
	'L'=>undef,		# RequestList
	'R'=>undef,		# ResponseList
	'A'=>undef,		# AKA
	'B'=>undef,		# Baud
	'T'=>undef		# Time
	);

$CurrentVersion=210;

%Config=(
	'VersionStamp'		=> $CurrentVersion,
	'Log'				=> '-',
	'DebugLog'			=> 0,
	'TempDir'			=> 'C:\\WINDOWS\\TEMP\\',
	'OurAKA'			=> [2,5030,661,14,''],
	'OurSysop'			=> 'Pavel Kostromitinov',

	'BinaryFile'		=> [],
	'OfflineFile'		=> [],
	'ScannedFiles'		=> [],
	'HistoryFile'		=> 'FREQLIM.BIN',

	'Sysop'				=> 'Sysop',
	'AKA'				=> [],
	'Baud'				=> 300,
	'Time'				=> -1,
	'RequestList'		=> undef,
	'ResponseList'		=> undef,
	'RemoteStatus'		=> 0,				# 0 for UNPROTECTED
	'SystemStatus'		=> 0,				# 0 for UNLISTED

	'SendStatistics'	=> 1,
    'ResponseHeader'	=> 'FREQHDR.TPL',
	'ResponseFooter'	=> 'FREQFTR.TPL',
	'ResponseSubject'	=> 'File Request results',
	'LineNormal'		=> ' %-13N  %+8S  %-45D ',
	'LineContinued'		=> '                          %-45D ',
	'LineNotFound'		=> ' %-13N            File not found or access denied               ',
	'LineTimeExceeded'	=> ' %-13N  %+8S  Download time limit exceeded                  ',
	'LineSizeExceeded'	=> ' %-13N  %+8S  Download size limit exceeded                  ',
	'LineOffline'		=> ' %-13N  %+8S  File is Offline, you should ask SysOp for it  ',
	'LineTooSlow'		=> '                          Connection too slow, sorry                    ',
	'LineUnlisted'		=> '                          Unlisted nodes not allowed, sorry             ',
	'LineUnprotected'	=> '                          Unprotected sessions not allowed, sorry       ',
	'LineDelimiter'		=> 'Ĵ',
	'DateTimeFormat'	=> '%>2_0D %M %Y  %{TM}',
	'Tearline'			=> 'FreqMgr/perl 2.10',
	'SendFilesBBS'		=> 1,
	'NormalPrefix'		=> '+',
	'KillSentPrefix'	=> '=',

	'CaseSensivity'		=> 2,	# Never/Always/LFNOnly/NonDOSOnly
	'KillREQ'			=> 0,

	'MinBaud'			=> 14400,
	'AllowOneLargeFile'	=> 1,
	'AllowUnlisted'		=> 1,
	'AllowUnprotected'	=> 1,
	'ListedLimit'		=> new FMgrLimit('',0,60,2048,8192,20480),
	'UnlistedLimit'		=> new FMgrLimit('',0,30,1024,4096,10240),
	'UnprotectedLimit'	=> new FMgrLimit('',0,60,2048,8192,20480),
	'UnknownLimit'		=> new FMgrLimit('',0,30,1024,4096,10240),
	'DirectFreqLevel'	=> 16,
	'PriorityLimit'		=> [],
	'PriorityAKA'		=> [],

	'Password'			=> {},
	'Alias'				=> {}
);

%Groups=(
	''=>0
);

$CurrentLimits=undef;

@TMailHistory=();
$CurrentHistory=undef;

sub Log {
	if (@_) {
		my @CTime=localtime(time());
		printf STDERR "%02d/%02d %02d:%02d:%02d ",$CTime[3]+1,$CTime[4]+1,$CTime[2],,$CTime[1],$CTime[0];
	}
	print STDERR @_,"\n";
}

sub LogDie {
	Log(@_);exit(1);
}

sub DebugLog {
	if ($Config{'DebugLog'}) {Log(@_);}
}

sub ParseAccess {
	my ($AccessString,$FailEmpty)=@_;
    $FailEmpty=0 unless defined $FailEmpty;
	return undef unless $AccessString=~/^\s*(\d+)?(@(\S*))?\s*$/;
	return undef if ($FailEmpty&&(!defined $3)&&(!defined $1));
	return ((defined $3)?$3:'',(defined $1)?$1:0);
}

sub CalculateLimits {
	my $Status;
	if ($Config{'SystemStatus'}) {
		$Status=($Config{'RemoteStatus'})?'Listed':'Unprotected';
	} else {
		$Status=($Config{'RemoteStatus'})?'Unlisted':'Unknown';
	}
    $Status=$Config{$Status.'Limit'};
	%$CurrentLimits=%$Status;
	foreach my $Limit (@{$Config{'PriorityLimit'}}) {
		if (exists($Groups{$Limit->{'Group'}})&&
		   ($Limit->{'Level'}<=$Groups{$Limit->{'Group'}})) {
			AddLimit($Limit);
		}
	}
}

sub Max {
	my ($A,$B)=@_;
	return ($A>$B)?$A:$B;
}

sub AddLimit {
	my $Limit=shift;
	$CurrentLimits->{'MaxTime'}=Max($CurrentLimits->{'MaxTime'},$Limit->{'MaxTime'});
	$CurrentLimits->{'DayLimit'}=Max($CurrentLimits->{'DayLimit'},$Limit->{'DayLimit'});
	$CurrentLimits->{'WeekLimit'}=Max($CurrentLimits->{'WeekLimit'},$Limit->{'WeekLimit'});
	$CurrentLimits->{'MonthLimit'}=Max($CurrentLimits->{'MonthLimit'},$Limit->{'MonthLimit'});
}

sub AddGroupAndLevel {
	my ($Group,$Level)=@_;
	$Groups{$Group}=$Level unless exists($Groups{$Group})&&($Groups{$Group}>$Level);
}

sub AddPassword {
	my ($Password,$IsSRIF)=@_;
	if (exists($Config{'Password'}->{$Password})) {
		$Password=$Config{'Password'}->{$Password};
		AddGroupAndLevel($Password->[0],$Password->[1]);
	} else {
		die "Undefined password: '$1'\n" unless $IsSRIF;
	}
}

1;
