# FreqMgr 2.10 - Configuration routines
# Copyright (C) 2.10, Kostromitinov Pavel
#
package FMgrConfig;
require Exporter;
@ISA=qw(Exporter);
@EXPORT=qw(ProcessCommandLine LoadConfig AnalyzeString);

use FMgrCommon;
use FMgrAlias;
use FMgrBinary;
use FMgrLimit;
use FMgrScan;
use FNdxCommon;
use PKConfig;
use PKParams;
use FTN;
BEGIN {do 'FMgrExport.pm';}

sub ProcessCommandLine {
	$Path=(ParseFile($0))[1];
	$StringParams{'C'}=$Path."FreqMgr.cfg";

	$Result=ParseCommandLine(\%SwitchParams,\%StringParams);
	die "Bad command-line parameter: $Result\n" if defined $Result;
	ShowHelp(),exit(0) unless (@ARGV<2);

	if ($SwitchParams{'H'}) {ShowHelp();exit(0);};

	LoadConfig($StringParams{'C'},0);
	foreach $String (@{$StringParams{'I'}}) {LoadConfig($String,0);}
	foreach $String (@{$StringParams{'S'}}) {AnalyzeString($String);}
	AnalyzeString('AKA '.$StringParams{'A'},1) if defined $StringParams{'A'};
	AnalyzeString('Baud '.$StringParams{'B'},1) if defined $StringParams{'B'};
	AnalyzeString('Time '.$StringParams{'T'},1) if defined $StringParams{'T'};
	AnalyzeString('RequestList '.$StringParams{'L'},1)  if defined $StringParams{'L'};
	AnalyzeString('ResponseList '.$StringParams{'R'},1)  if defined $StringParams{'R'};
	LoadConfig($ARGV[0],1) if (@ARGV);

	close(STDERR);
	die "Cannot open log file '",$Config{'Log'},"': $!\n" unless open(STDERR,'>'.$Config{'Log'});
}

# LoadConfig(FileName,IsSRIF)
sub LoadConfig {
	my ($FileName,$IsSRIF)=@_;
	local *CONFIG;
	open(CONFIG,$FileName) or LogDie("Cannot open '$FileName':$!");
	while (defined (my $Line=ReadConfigLine(*CONFIG{IO}))) {
		AnalyzeString($Line,$IsSRIF);
	}
	close(CONFIG);
}

# AnalyzeString(Line,IsSRIF)
sub AnalyzeString {
	my ($String,$IsSRIF)=@_;
	my ($UString,$Value);

	($String,$Value,undef)=ParseConfigLine($String);
	return if $String eq '';
	LogDie("Invalid configuration line: '$String'") unless defined $Value;
	$UString=uc($String);

   	if ($UString eq 'VERSIONSTAMP') {
   		if ($Value=~/^(\d+)\.(\d+)$/) {
			$Config{'VersionStamp'}=$1*100+$2;
			LogDie("This configuration file is for a later version of the script")
				if $Config{'VersionStamp'}>$CurrentVersion;
   			return;
   		} else {
   			LogDie("Invalid version stamp: '$Value'");
		}
   	}
	if ($UString eq 'LOG') {$Config{'Log'}=FileNew($Value);return;};
	if ($UString eq 'DEBUGLOG') {$Config{'DebugLog'}=Boolean($Value);return;};
	if ($UString eq 'TEMPDIR') {$Config{'TempDir'}=StripQuotes($Value);return;};
	if ($UString eq 'OURAKA') {
		my @OurAKA=String2Address($Value);
		LogDie("Invalid address string: '$Value'") unless @OurAKA;
		$Config{'OurAKA'}=\@OurAKA;
		return;
	};
	if ($UString eq 'OURSYSOP') {$Config{'OurSysop'}=StripQuotes($Value);return;}
	if ($UString eq 'BINARYFILE') {
		my @Words=Words($Value);
		$Value=shift @Words;
		push(@{$Config{'BinaryFile'}},load FMgrBinary(FileExists($Value),\@Words));
		return;
	}
	if ($UString eq 'OFFLINEFILE') {
		my @Words=Words($Value);
		$Value=shift @Words;
		push(@{$Config{'OfflineFile'}},load FMgrBinary(FileExists($Value),\@Words));
		return;
	}
	if (($UString eq 'SCANDIRECTORY')||($UString eq 'RECURSEDIRECTORY')) {
		my $Path=StripWord(\$Value);
		my @Access=ParseAccess($Value);
		my $Areas;
		if ($UString eq 'SCANDIRECTORY') {
			LogDie("Invalid ScanDirectory string: '$Path $Value'") unless @Access;
			my $Area=ScanDirectory($Path,$Access[1]);
			$Areas=[$Area];
		} else {
			LogDie("Invalid RecurseDirectory string: '$Path $Value'") unless @Access;
			$Areas=RecurseDirectory($Path,$Access[1]);
		}
		my $Binary=new FMgrBinary($Areas,[$Access[0]]);
		push(@{$Config{'ScannedFiles'}},$Binary);
		return;
	}

	if ($UString eq 'HISTORYFILE') {$Config{'HistoryFile'}=FileNew($Value);return;}

	if ($UString eq 'SYSOP') {$Config{'Sysop'}=StripQuotes($Value);return;}
	if ($UString eq 'AKA') {
		my @AKA=String2Address($Value);
		LogDie("Invalid address string: '$Value'") unless @AKA;
		push(@{$Config{'AKA'}},\@AKA);
		return;
	}
	if ($UString eq 'BAUD') {$Config{'Baud'}=Number($Value);return;}
	if ($UString eq 'TIME') {$Config{'Time'}=($Value==-1)?-1:Number($Value);return;}
	if ($UString eq 'REQUESTLIST') {
		$Config{'RequestList'}=($IsSRIF)?FileExists($Value):$Value;
		return;
	}
	if ($UString eq 'RESPONSELIST') {$Config{'ResponseList'}=FileNew($Value);return;}
	if ($UString eq 'REMOTESTATUS') {
		if (uc($Value) eq 'PROTECTED') {$Config{'RemoteStatus'}=1;return;}
		if (uc($Value) eq 'UNPROTECTED') {$Config{'RemoteStatus'}=0;return;}
		LogDie("Invalid Remote Status: '$Value'");
	}
	if ($UString eq 'SYSTEMSTATUS') {
		if (uc($Value) eq 'LISTED') {$Config{'SystemStatus'}=1;return;}
		if (uc($Value) eq 'UNLISTED') {$Config{'SystemStatus'}=0;return;}
		LogDie("Invalid System Status: '$Value'");
	}

	if ($UString eq 'KILLREQ') {$Config{'KillREQ'}=Boolean($Value);return;}
	if ($UString eq 'SENDSTATISTICS') {$Config{'SendStatistics'}=Boolean($Value);return;}
	if ($UString eq 'RESPONSEHEADER') {$Config{'ResponseHeader'}=FileExists($Value);return;}
	if ($UString eq 'RESPONSEFOOTER') {$Config{'ResponseFooter'}=FileExists($Value);return;}
	if ($UString eq 'RESPONSESUBJECT') {$Config{'ResponseSubject'}=StripQuotes($Value);return;}
	if ($UString eq 'LINENORMAL') {$Config{'LineNormal'}=StripQuotes($Value);return;}
	if ($UString eq 'LINECONTINUED') {$Config{'LineContinued'}=StripQuotes($Value);return;}
	if ($UString eq 'LINENOTFOUND') {$Config{'LineNotFound'}=StripQuotes($Value);return;}
	if ($UString eq 'LINETIMEEXCEEDED') {$Config{'LineTimeExceeded'}=StripQuotes($Value);return;}
	if ($UString eq 'LINESIZEEXCEEDED') {$Config{'LineSizeExceeded'}=StripQuotes($Value);return;}
	if ($UString eq 'LINEOFFLINE') {$Config{'LineOffline'}=StripQuotes($Value);return;}
	if ($UString eq 'LINETOOSLOW') {$Config{'LineTooSlow'}=StripQuotes($Value);return;}
	if ($UString eq 'LINEUNLISTED') {$Config{'LineUnlisted'}=StripQuotes($Value);return;}
	if ($UString eq 'LINEUNPROTECTED') {$Config{'LineUnprotected'}=StripQuotes($Value);return;}
	if ($UString eq 'LINEDELIMITER') {$Config{'LineDelimiter'}=StripQuotes($Value);return;}
	if ($UString eq 'DATETIMEFORMAT') {$Config{'DateTimeFormat'}=StripQuotes($Value);return;}
	if ($UString eq 'TEARLINE') {$Config{'Tearline'}=StripQuotes($Value);return;}

	if ($UString eq 'SENDFILESBBS') {$Config{'SendFilesBBS'}=Boolean($Value);return;}
	if ($UString eq 'NORMALPREFIX') {$Config{'NormalPrefix'}=StripQuotes($Value);return;}
	if ($UString eq 'KILLSENTPREFIX') {$Config{'KillSentPrefix'}=StripQuotes($Value);return;}
	if ($UString eq 'CASESENSIVITY') {
		my $UCValue=uc($Value);
		if (defined Boolean($Value,1)) {$Config{'CaseSensivity'}=Boolean($Value);return;}
		if ($UCValue eq 'NEVER') {$Config{'CaseSensivity'}=0;return;}
		if ($UCValue eq 'ALWAYS') {$Config{'CaseSensivity'}=1;return;}
		if (($UCValue eq 'LFN')||($UCValue eq 'LFNONLY')) {$Config{'CaseSensivity'}=2;return;}
		if (($UCValue eq 'NONDOS')||($UCValue eq 'NONDOSONLY')) {$Config{'CaseSensivity'}=3;return;}
		LogDie("Invalid CaseSensivity: '$Value'");
	}

	if ($UString eq 'MINBAUD') {$Config{'MinBaud'}=Number($Value);return;}
	if ($UString eq 'ALLOWONELARGEFILE') {$Config{'AllowOneLargeFile'}=Boolean($Value);return;}
	if ($UString eq 'ALLOWUNLISTED') {$Config{'AllowUnlisted'}=Boolean($Value);return;}
	if ($UString eq 'ALLOWUNPROTECTED') {$Config{'AllowUnprotected'}=Boolean($Value);return;}

	if ($UString eq 'LISTEDLIMIT') {$Config{'ListedLimit'}=FromLine FMgrLimit($Value);return;}
	if ($UString eq 'UNLISTEDLIMIT') {$Config{'UnlistedLimit'}=FromLine FMgrLimit($Value);return;}
	if ($UString eq 'UNPROTECTEDLIMIT') {$Config{'UnprotectedLimit'}=FromLine FMgrLimit($Value);return;}
	if ($UString eq 'UNKNOWNLIMIT') {$Config{'UnknownLimit'}=FromLine FMgrLimit($Value);return;}
	if ($UString eq 'PRIORITYLIMIT') {push(@{$Config{'PriorityLimit'}},FromLine FMgrLimit($Value));return;}
    if ($UString eq 'PRIORITYAKA') {
    	my $AKA=StripWord(\$Value);
		my @AKA=String2AddressMask($AKA);
		LogDie("Invalid address mask string: '$Value'") unless @AKA;
		my @Access=ParseAccess($Value,1);
		LogDie("Invalid PriorityAKA string: '$AKA $Value'") unless @Access;
		push(@{$Config{'PriorityAKA'}},{'AKA'=>\@AKA,'Group'=>$Access[0],'Level'=>$Access[1]});
		return;
    }

	if ($UString eq 'DIRECTFREQLEVEL') {$Config{'DirectFreqLevel'}=Number($Value);return;}

	if ($UString eq 'PASSWORD') {
		my $Password=StripWord(\$Value);
		if ($Value=~/^\s*$/) {AddPassword($Password,$IsSRIF);return;}
		my @Access=ParseAccess($Value);
		LogDie("Invalid Password string: '$Password $Value'") unless @Access;
		$Config{'Password'}->{$Password}=\@Access;
		return;
	}
	if ($UString eq 'ALIAS') {
		if      ($Value=~/^(\w+)\s+((Oldest|Latest|Newest|All)\s+)?(\S+|(\".*?\"))\s*(.*?)\s*$/i) {
			my $Alias=new FMgrAlias($1,(defined $5)?$5:$4,$6,(defined $3)?$3:'');
			$Config{'Aliases'}->{uc($1)}=$Alias;
		} elsif ($Value=~/^(\w+)\s*=\s*(\w+)\s*$/) {
			if (exists $Config{'Aliases'}->{uc($2)}) {
				$Config{'Aliases'}->{uc($1)}=$Config{'Aliases'}->{uc($2)};
			} else {
				LogDie("Undefined alias: '$2'");
			}
		} else {
			LogDie("Invalid Alias string: '$String $Value'");
		}
		return;
	}

    if ($UString eq 'INCLUDE') {LoadConfig($Value);return;};
	LogDie("Unrecognized keyword: '$String'") unless $IsSRIF;
}

sub ShowHelp {
	print <<'EOH';
SRIF-compatible Freq manager version 2.10/perl
Usage: FreqMgr.pl [options] [SRIF file]
Options:
	-Cfile        - Read configuration from 'file' (FreqMgr.cfg by default)
	-Ifile        - Include 'file' to configuration
	-Sstring      - Add 'string' to configuration
	-Aaka         - Remote AKA
	-Bbaud        - Commection speed
	-Ttime        - Maximal time to transmit files
	-Lreqlist     - Requested files list
	-Rresplist    - Response list
	-H            - This help
EOH
}

1;
