# FTNPacket class
#
# Currently implemented:
# Writing Type 2.0+ packet [FSC-0039]
#
# Copyright (C) 2.10, Kostromitinov Pavel
#
package FTNPacket;
use FTN;

sub new {
	my ($Class)=@_;
	$Class=ref $Class if ref $Class;
	my %FTNPacket=(
		'OrigAddr'		=>	[0,0,0,0,''],
		'DestAddr'		=>	[0,0,0,0,''],
		'Time'			=>	time(),
		'Baud'			=>	33600,
		'Version'		=>	2,		# Do not modify
		'ProductCode'	=>	0xFFFF,
		'ProductRev'	=>	0,
		'Password'		=>	'',
		'Filler'		=>	0,
		'CapWord'		=>	1,
		'UserData'		=>	0,
		'Terminator'	=>	0		# Do not modify
	);
	return bless \%FTNPacket,$Class;
}

sub Header {
	my ($This)=@_;
	my @Time=localtime($This->{'Time'});
	return pack('vvvvvvvvvvvvCCZ8vvvnCCvvvvvvv',
		$This->{'OrigAddr'}->[2],
		$This->{'DestAddr'}->[2],
		$Time[5]+1900,$Time[4],$Time[3],
		$Time[2],$Time[1],$Time[0],
        $This->{'Baud'},
		$This->{'Version'},
		$This->{'OrigAddr'}->[1],
		$This->{'DestAddr'}->[1],
		$This->{'ProductCode'}&0x00FF,
		($This->{'ProductRev'}&0xFF00)>>8,
		$This->{'Password'},
		$This->{'OrigAddr'}->[0],
		$This->{'DestAddr'}->[0],
		$This->{'Filler'},
		$This->{'CapWord'},
		($This->{'ProductCode'}&0xFF00)>>8,
		$This->{'ProductRev'}&0x00FF,
		$This->{'CapWord'},
		$This->{'OrigAddr'}->[0],
		$This->{'DestAddr'}->[0],
		$This->{'OrigAddr'}->[3],
		$This->{'DestAddr'}->[3],
		$This->{'UserData'},
		$This->{'Terminator'}
		);
}

1;

package FTNPackedMsg;
use FTN;

sub new {
	my ($Class)=@_;
	$Class=ref $Class if ref $Class;
	my %FTNPackedMsg=(
		'OrigNode'		=>	0,
		'DestNode'		=>	0,
		'OrigNet'		=>	0,
		'DestNet'		=>	0,
		'Attributes'	=>	0,
		'Cost'			=>	0,
		'Time'			=>	time(),
		'FromName'		=>	'',
		'ToName'		=>	'',
		'Subject'		=>	'',
		'Text'			=>	[]
	);
	return bless \%FTNPackedMsg,$Class;
}

sub Binary {
	my ($This)=@_;
	my @Month3=('Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec');
	my @Time=localtime($This->{'Time'});
	my $Header=pack('CCvvvvvvA20',0x02,0x00,
		$This->{'OrigNode'},
		$This->{'DestNode'},
		$This->{'OrigNet'},
		$This->{'DestNet'},
		$This->{'Attributes'},
		$This->{'Cost'},
		sprintf('%02d %s %02d  %02d:%02d:%02d',$Time[3],$Month3[$Time[4]],$Time[5] % 100,$Time[2],$Time[1],$Time[0]),
	);
	$Header.=$This->{'ToName'}."\x00";
	$Header.=$This->{'FromName'}."\x00";
	$Header.=$This->{'Subject'}."\x00";
	foreach my $Line (@{$This->{'Text'}}) {
		$Header.=$Line."\x0D";
	}
	return $Header."\x00";

}

sub SetOrigAddr {
	my ($This,$OrigAddr)=@_;
	$This->{'OrigNet'}=$OrigAddr->[1];
	$This->{'OrigNode'}=$OrigAddr->[2];
}

sub SetDestAddr {
	my ($This,$DestAddr)=@_;
	$This->{'DestNet'}=$DestAddr->[1];
	$This->{'DestNode'}=$DestAddr->[2];
}

sub AddKludgeLines {
	my ($This,$Orig,$Dest)=@_;
	my @Lines=KludgeLines($Orig,$Dest);
	push(@{$This->{'Text'}},@Lines);
}

1;
