#!/usr/local/bin/perl -w
#
# FreqMgr 2.10 - SRIF-compatible File Request manager
# Copyright (C) 2.10, Kostromitinov Pavel
#
BEGIN {
	my $ScriptPath=$0;
	$ScriptPath=~/^(.*)[\/\\:]/;		# Search for modules in current folder
	if ((defined $1)&&($1 ne '')) {
		$ScriptPath=$1;push (@INC,$ScriptPath);
		$ScriptPath=~/^(.*)[\/\\:]/;	# Search for modules in upper folder
		push (@INC,$1) if ((defined $1)&&($1 ne ''));
	}
}

use FNdxCommon;
use FMgrCommon;
use FMgrConfig;
use PKParams;
use PKFormat;
use PKConfig;
use FTN;
use FTNPacket;
use Data::Dumper;
use File::Basename;
use File::Spec;
BEGIN {do 'FMgrExport.pm';}

ProcessCommandLine();
Log("FreqMgr.pl started");

LogDie("No AKAs for remote defined!") unless @{$Config{'AKA'}};
Log('Session: ',Address2String($Config{'AKA'}->[0]),
	(($Config{'SystemStatus'})?' ':' UN'),"LISTED, ",
	(($Config{'RemoteStatus'})?'':'UN'),"PROTECTED");
Log('Connect: ',$Config{'Baud'},' baud, ',$Config{'Time'}," min left");

foreach my $PrioAKA (@{$Config{'PriorityAKA'}}) {
	foreach my $AKA (@{$Config{'AKA'}}) {
		if (AddressMask($PrioAKA->{'AKA'},$AKA)) {
			AddGroupAndLevel($PrioAKA->{'Group'},$PrioAKA->{'Level'});
			last;
		}
	}
}

if (open(HISFILE,"<$Config{'HistoryFile'}")) {
	binmode HISFILE;
	my $Record;
	while (read(HISFILE,$Record,0x48)==0x48) {
		my @Days=unpack('v4v30V',$Record);
		my @Addr=splice(@Days,0,4);push(@Addr,'');
		my $LastDate=pop(@Days);
		my $History=new TMailHistory(\@Addr,\@Days,$LastDate);
		push(@TMailHistory,$History);
	};
	close(HISFILE);
}

foreach my $AKA (@{$Config{'AKA'}}) {
	foreach my $History (@TMailHistory) {
		if (AddressMask($AKA,$History->{'Addr'})) {
			$CurrentHistory=$History;last;
		}
	}
	last if defined $CurrentHistory;
}

unless (defined $CurrentHistory) {
	$CurrentHistory=new TMailHistory($Config{'AKA'}->[0],[],time());
	push(@TMailHistory,$CurrentHistory);
}

open(REQFILE,"<$Config{'RequestList'}") or die "Cannot open $Config{'RequestList'}: $!\n";
open(RESFILE,">$Config{'ResponseList'}") or die "Cannot create $Config{'ResponseList'}: $!\n";

$TempFile=$Config{'TempDir'}.sprintf("%08x.pkt",time()^$$);
open(PKTFILE,">$TempFile") or die "Cannot create '$TempFile': $!\n";
binmode PKTFILE;

$Packet=new FTNPacket();
$Message=new FTNPackedMsg();
print RESFILE $Config{'KillSentPrefix'}.$TempFile."\n";

my ($TotalRequests,$TotalFiles,$TotalSize)=(0,0,0);
# Normal, Limited, Not found
my @Responses=([],[],[]);

$FatalError=0;
if ($Config{'Baud'}<$Config{'MinBaud'}) {
	$FatalError=1;
	push(@{$Responses[2]},$Config{'LineTooSlow'});
}
if ((scalar keys %Groups==1)&&($Groups{''}==0)) {
	if (($Config{'SystemStatus'}==0)&&($Config{'AllowUnlisted'}==0)) {
		$FatalError=1;
		push(@{$Responses[2]},$Config{'LineUnlisted'});
	}
	if (($Config{'RemoteStatus'}==0)&&($Config{'AllowUnprotected'}==0)) {
		$FatalError=1;
		push(@{$Responses[2]},$Config{'LineUnprotected'});
	}
}

unless ($FatalError) {
	while (defined(my $Requested=<REQFILE>)) {
		chomp $Requested;
		ProcessRequest($Requested);
	}
	$CurrentHistory->{'Days'}->[0]+=$TotalSize>>10;
}

Log();WriteMessage();

close(REQFILE);
close(RESFILE);
close(PKTFILE);

if ($Config{'KillREQ'}) {
	print "Cannot delete $Config{'RequestList'}: $!\n" unless unlink($Config{'RequestList'});
}

if (open(HISFILE,">$Config{'HistoryFile'}")) {
	binmode HISFILE;
	foreach my $History (@TMailHistory) {
		print HISFILE $History->Binary();
	}
	close(HISFILE);
} else {
	Log("Warning! Cannot create history file '$Config{'HistoryFile'}': $!");
}

Log("FreqMgr.pl finished");

# --------------------------

sub ProcessRequest {
	my ($Requested)=@_;
	CalculateLimits();
	Log();
	Log("Request: '$Requested'");
	if (exists($Config{'Password'}->{$Requested})) {
		DebugLog("'$Requested' recognized as password");
		AddPassword($Requested,1);return;
	}

	$TotalRequests++;

	if (exists($Config{'Aliases'}->{uc($Requested)})) {
		DebugLog("'$Requested' recognized as alias");
		ProcessAlias($Config{'Aliases'}->{uc($Requested)});return;
	}

	my $Found=0;
	if (File::Spec->file_name_is_absolute($Requested)) {
		DebugLog("'$Requested' recognized as direct freq");
		if ($Groups{''}>=$Config{'DirectFreqLevel'}) {
			$Found=SendFile($Requested,['']);
		} else {
			Log("Access level too low for direct freq");
		}
	} else {
		foreach my $Binary (@{$Config{'BinaryFile'}}) {$Found|=ScanBinary($Binary,$Requested,0);}
		foreach my $Binary (@{$Config{'OfflineFile'}}) {$Found|=ScanBinary($Binary,$Requested,1);}
		foreach my $Binary (@{$Config{'ScannedFiles'}}) {$Found|=ScanBinary($Binary,$Requested,0);}
	}

	unless ($Found) {
		my %Params=(
			'N'	=> $Requested
		);
	    push(@{$Responses[2]},ExpandString($Config{'LineNotFound'},\%Params));
		Log("'$Requested' unrecognized");
	}
}

sub ProcessAlias {
	my ($Alias)=@_;
	my ($Mask,$Path)=ParseFile($Alias->{'File'});
	$Path=$Volume.$Path if defined $Volume;
	my @Files;

	return unless (opendir DIRHANDLE,$Path);
	while (defined (my $FileName=readdir DIRHANDLE)) {
		if (IsFileMask($FileName,$Mask)) {
			$FileName=$Path.$FileName;
			push(@Files,$FileName) if (-f $FileName);
		}
	}
	closedir DIRHANDLE;
    return unless @Files;

	my @Times=map {(stat($_))[9]} @Files;
	if ($Alias->{'Which'}==1) {
		foreach my $File (@Files) {SendFile($File,[$Alias->{'Description'}]);}
		return;
	}

	my ($Needed,$I)=0;
	for ($I=0;$I<@Times;$I++) {
		$Needed=$I if (($Times[$I]<=>$Times[$Needed])+1==$Alias->{'Which'})
	}
	SendFile($Files[$Needed],[$Alias->{'Description'}]);
}

sub IsFileMaskCaseDep {
	my ($File,$Mask)=@_;
	my $CaseSensitive=0;
	$CaseSensitive=1 if ($Config{'CaseSensivity'}==1);
    if ($Config{'CaseSensivity'}==2) {
    	$CaseSensitive=(length($Mask)>12)?1:0;
    }
    if ($Config{'CaseSensivity'}==3) {
    	$CaseSensitive=($Mask=~/^.{1,8}(\..{0,3}|)$/)?0:1;
    }
    return ($CaseSensitive)?IsFileMaskWithCase($File,$Mask):IsFileMask($File,$Mask);
}

sub ScanBinary {
	my ($Binary,$Requested,$Offline)=@_;
	my $Found=0;

	my $EffectiveLevel=-1;
	foreach my $Group (@{$Binary->{'Groups'}}) {
		if (exists($Groups{$Group})) {
			$EffectiveLevel=$Groups{$Group} if $Groups{$Group}>$EffectiveLevel;
		}
	}
	return 0 unless $EffectiveLevel>=0;
	foreach my $Area (@{$Binary->{'Areas'}}) {
		next unless $EffectiveLevel>=$Area->{'AccessLevel'};
		foreach my $File (@{$Area->{'Files'}}) {
			if (IsFileMaskCaseDep($File->{'Name'},$Requested)) {
				my $OfflineSize=(($Offline)||($Area->{'Type'}==2))?$File->{'Size'}:undef;
				$Found|=SendFile($Area->{'Path'}.$File->{'Name'},$File->{'Description'},$OfflineSize);
			}
		}
	}
	return $Found;
}

sub SendFile {
	my ($FileName,$Description,$OfflineSize)=@_;
    $Description=[''] unless defined $Description;

	my %Params=(
		'N'	=> basename($FileName),
		'S'	=> $OfflineSize,
		'D'	=> $Description->[0]
	);

	if (defined($OfflineSize)) {
		push(@{$Responses[2]},ExpandString($Config{'LineOffline'},\%Params));
		Log("'$FileName' is offline");
		return 1;
    }

	my $Size=(stat($FileName))[7];
	return 0 unless defined $Size;

	my $AllSizeK=($TotalSize+$Size)/1024;
	my $Time=($TotalSize+$Size)*8/($Config{'Baud'}*60);
	$Params{'S'}=$Size;

	if (($TotalSize>0)||(!$Config{'AllowOneLargeFile'})) {

		if (($AllSizeK+$CurrentHistory->{'Days'}->[0]>$CurrentLimits->{'DayLimit'})||
			($AllSizeK+$CurrentHistory->ThisWeek()>$CurrentLimits->{'WeekLimit'})||
			($AllSizeK+$CurrentHistory->ThisMonth()>$CurrentLimits->{'MonthLimit'})) {
			push(@{$Responses[1]},ExpandString($Config{'LineSizeExceeded'},\%Params));
			Log("'$FileName' exceeds allowed size");
			return 1;
		}
		if ($Time>$CurrentLimits->{'MaxTime'}) {
			push(@{$Responses[1]},ExpandString($Config{'LineTimeExceeded'},\%Params));
			Log("'$FileName' exceeds allowed time");
			return 1;
		}
	}
	$TotalFiles++;
	$TotalSize+=$Size;
	print RESFILE $Config{'NormalPrefix'}.$FileName."\n";
	my $WhatLine='LineNormal';
	foreach my $DescLine (@$Description) {
		$Params{'D'}=$DescLine;
		push(@{$Responses[0]},ExpandString($Config{$WhatLine},\%Params));
		$WhatLine='LineContinued';
	}
	Log("Added: '$FileName'");
	return 1;
}

sub WriteResponse {
	my ($RespGroup,$NeedDelimiter)=@_;
	if (@{$Responses[$RespGroup]}) {
		push(@{$Message->{'Text'}},$Config{'LineDelimiter'}) if ($NeedDelimiter);
		push(@{$Message->{'Text'}},@{$Responses[$RespGroup]});
		return 1;
	} else {return 0;}
}

sub WriteMessage {
	CalculateLimits();
	$Packet->{'OrigAddr'}=$Config{'OurAKA'};
	$Packet->{'DestAddr'}=$Config{'AKA'}->[0];
	print PKTFILE $Packet->Header();

    my %Params=(
		'N'		=>	$Config{'Sysop'},
		'A'		=>	Address2String($Config{'AKA'}->[0]),
		'F'		=>	$TotalRequests,
		'O'		=>	$TotalFiles,
		'S'		=>	$TotalSize,
		'T'		=>	FormatDateTime(time(),$Config{'DateTimeFormat'}),

		'TL'	=>	$CurrentLimits->{'MaxTime'},
#		'TE'	=>	$Time,
		'DD'	=>	$CurrentHistory->{'Days'}->[0],
		'DL'	=>	$CurrentLimits->{'DayLimit'},
		'WD'	=>	$CurrentHistory->ThisWeek(),
		'WL'	=>	$CurrentLimits->{'WeekLimit'},
		'MD'	=>	$CurrentHistory->ThisMonth(),
		'ML'	=>	$CurrentLimits->{'MonthLimit'},
	);
	$Params{'DR'}=$Params{'DL'}-$Params{'DD'};
	$Params{'WR'}=$Params{'WL'}-$Params{'WD'};
	$Params{'MR'}=$Params{'ML'}-$Params{'MD'};

	$Message->SetOrigAddr($Config{'OurAKA'});
	$Message->SetDestAddr($Config{'AKA'}->[0]);
	$Message->{'FromName'}=$Config{'OurSysop'};
	$Message->{'ToName'}=$Config{'Sysop'};
	$Message->{'Subject'}=ExpandString($Config{'ResponseSubject'},\%Params);
	$Message->AddKludgeLines($Config{'OurAKA'},$Config{'AKA'}->[0]);

	push(@{$Message->{'Text'}},@{TemplateFromFile($Config{'ResponseHeader'},\%Params)});

	my $NeedDelimiter=WriteResponse(0,0);
	$NeedDelimiter|=WriteResponse(1,$NeedDelimiter);
	WriteResponse(2,$NeedDelimiter);

	push(@{$Message->{'Text'}},@{TemplateFromFile($Config{'ResponseFooter'},\%Params)});
	push(@{$Message->{'Text'}},'','--- '.$Config{'Tearline'});

	print PKTFILE $Message->Binary();
	print PKTFILE "\x00\x00";
}
