# Config file parsing
# Copyright (C) 2000, Kostromitinov Pavel
#
package PKConfig;
require Exporter;
@ISA=qw(Exporter);
@EXPORT=qw(ReadConfigLine ParseConfigLine StripComment ParseFile CreateDirForFile
		CreateDir Number Boolean FileExists FileNew Word StripWord Words StripQuotes);
@EXPORT_OK=qw();
use Carp;
use File::Basename;

# Parameters; IOref
# Returns ($ParamName $ParamValue)
sub ReadConfigLine {
	my ($FileRef)=@_;
	my $Line;

	$Line=<$FileRef>;
	return undef unless defined $Line;
	$Line=StripComment($Line);
	while ($Line=~/^(.*)(?<!\\)\\\s*$/) {
		$Line=$1;
		my $NextLine=<$FileRef>;
		if (defined $NextLine) {
			$Line.=StripComment($NextLine);
		};
	}
	$Line=$1 if ($Line=~/^(.*)\\\s*$/);
	return $Line;
}

sub ParseConfigLine {
	my $Line=shift;
	return ('') if $Line=~/^\s*$/;
	return ($Line) unless $Line=~/^(\w+)(\s+(.*?)\s?)?$/;
	return ($1,defined($3)?$3:'',$Line);
}

sub StripComment {
	my $Line=shift;
	chomp $Line;
	$Line=~s/\s*(?<!\\)[;#].*$//;
	$Line=~s/\\([;#])/$1/g;
	return $Line;
}

#	Returns 0:File.Ext 1:Path WITH Slash
sub ParseFile {
	my $File=shift;
	if ($File=~/^.:$/) {
		return ($File,'');
	} else {
		my @Parse=fileparse($File);
		$Parse[0]=$Parse[0].$Parse[2] if defined $Parse[2];
		return ($Parse[0],$Parse[1]);
	}
}

sub CreateDirForFile {
	my $Path=shift;
	my $SubPath=(ParseFile($Path))[1];
	chop $SubPath;
	$SubPath=substr($SubPath,1) if $SubPath=~/^>/;
	return CreateDir($SubPath);
}

sub CreateDir {
	my $Path=shift;
	my $SubPath=(ParseFile($Path))[1];
	chop $SubPath;
	return 0 if ($Path eq '');
	return 1 if (-d $Path);
	return 0 if ($SubPath eq $Path);
	return 0 unless CreateDir($SubPath);
	return mkdir($Path,0777);
}

sub Number {
	my $Value=shift;
	return $Value if $Value=~/^\d+$/;
	die("'$Value' is not a valid number.\n");
}

sub Boolean {
	my $Value=uc($_[0]);
	if (($Value eq 'ON')|($Value eq 'YES')|($Value eq '1')) {return 1;}
	if (($Value eq 'OFF')|($Value eq 'NO')|($Value eq '0')) {return 0;}
	if (defined $_[1]) {return undef;} else {die("'$Value' is not a valid boolean.\n");}
}

sub FileExists {
	my $Value=shift;
	return $Value if (-f $Value);
	die("File not found: '$Value'.\n");
}

sub FileNew {
	my $Value=shift;
	return $Value if CreateDirForFile($Value);
	die("Cannot create directories for file '$Value'.\n");
}

sub Word {
	my $Value=shift;
	return StripWord(\$Value);
}

sub StripWord {
	my $Value=shift;
	return '' unless defined $Value;
	croak "Not a scalar-ref passed" if ref($Value) ne 'SCALAR';
    return '' unless $$Value=~/^\s*("(.*)"|\S+)\s*(.*)$/;
	$$Value=$3;
	return (defined $2)?$2:$1;
}

sub Words {
	my $Value=shift;
	my @Words=();
	while ((my $NextWord=StripWord(\$Value)) ne '') {push(@Words,$NextWord);}
	return @Words;
}

sub StripQuotes {
	my $Value=shift;
	return $1 if ($Value=~/\s*"(.*)"\s*/);
	return $1 if ($Value=~/\s*'(.*)'\s*/);
	return $1if ($Value=~/\s*`(.*)`\s*/);
	return $Value;
}

1;
