# String formatting routines
# Copyright (C) 2000, Kostromitinov Pavel
#
package PKFormat;
require Exporter;
use Carp qw(cluck);
@ISA=qw(Exporter);
@EXPORT=qw(ExpandString ExpandStringNoCR AlignString FormatDateTime
		Template TemplateFromFile PrintTemplate PrintTemplateFromFile
		IsFileMask IsFileMaskWithCase);

sub ExpandString {
	my ($String,$Params)=@_;
	my $Result='';
	croak("Not a hash-ref passed") if (ref $Params ne 'HASH');
	while ($String=~/(.*)%\\(.*)/) {
		$Result=$Result.ExpandStringNoCR($1,$Params)."\n";
		$String=$2;
	}
	return $Result.ExpandStringNoCR($String,$Params);
}

# %[[-|=|+]ddd][!][_c]c
# last 'c' can be either one char or {string}
sub ExpandStringNoCR {
	my ($String,$Params)=@_;
	my $Result='';
	croak("Not a hash-ref passed") if (ref $Params ne 'HASH');

	while ($String=~/(.*?)%((-|<|=|\+|>)?(\d+))?(!)?(_(.))?(\{(.*?)\}|.)(.*)$/s) {
		my ($AlignBy,$AlignTo,$FillChar,$Exact,$Param,$Value)=(0,0,' ',0,0,'');
		$Param=(defined $9)?$9:$8;
		$Result.=$1;
		if (defined $3) {
			if (($3 eq '-')||($3 eq '<')) {$AlignTo=0;} else {
			if ($3 eq '=') {$AlignTo=1;} else {$AlignTo=2;}
			}
		}
		$AlignBy=$4 if defined $4;
		$Exact=1 if defined $5;
		$FillChar=$7 if defined $7;
		if ($Param eq '%') {$Value='%';} else {
			if (defined $Params->{$Param}) {
				$Value=$Params->{$Param};
			} else {
				$Value='';
			}
		}
		if ($Param eq '.') {
			$Result=AlignString($Result,$AlignBy,$AlignTo,$FillChar,$Exact);
		} else {
			$Result.=AlignString($Value,$AlignBy,$AlignTo,$FillChar,$Exact);
		}
		$String=$10;
	}
	return $Result.$String;
}

# Align(String,AlignBy,AlignTo,FillChar,Exact)
sub AlignString {
	my ($String,$AlignBy,$AlignTo,$FillChar,$Exact)=@_;
	$AlignBy=0 unless defined $AlignBy;
	$AlignTo=0 unless defined $AlignTo;
	$FillChar=' ' unless defined $FillChar;
	$Exact=0 unless defined $Exact;
	if (length($String)>=$AlignBy) {
		if ($Exact) {
			if ($AlignTo==0) {return substr($String,0,$AlignBy);}
			if ($AlignTo==1) {$String=substr($String,0,(length($String)+$AlignBy)/2);}
			return substr($String,-$AlignBy);
		} else {return $String;}
	} else {
		if ($AlignTo==0) {return $String.($FillChar x ($AlignBy-length($String)));}
		if ($AlignTo==1) {
			$String.=($FillChar x (($AlignBy-length($String))/2));
		}
		{return ($FillChar x ($AlignBy-length($String))).$String;}
	}
}

sub FormatDateTime {
	my ($DateTime,$Format)=@_;
	my @Time=localtime($_[0]);
	my %DateParams=(
		'D'=>$Time[3],
		'M'=>('Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec')[$Time[4]],
		'ML'=>('January','February','March','April','May','June','July','August',
			'September','October','November','December')[$Time[4]],
		'N'=>$Time[4]+1,
		'O'=>('I','II','III','IV','V','VI','VII','VIII','IX','X','XI','XII')[$Time[4]],
		'Y'=>$Time[5]+1900,
		'R'=>$Time[5]%100,
		'H'=>$Time[2],
		'I'=>$Time[1],
		'S'=>$Time[0],

		'DT'	=>	sprintf('%02d.%02d.%04d',$Time[3],$Time[4]+1,$Time[5]+1900),
		'TM'	=>	sprintf('%02d:%02d:%02d',$Time[2],$Time[1],$Time[0])
	);
	return ExpandString($Format,\%DateParams);
}

sub Template {
	my ($Template,$Params)=@_;
	my @Result;

	foreach my $Line (@$Template) {
		next if substr($Line,0,1) eq ';';
		push(@Result,ExpandString($Line,$Params));
	}
	return \@Result;
}

sub TemplateFromFile {
	my ($FileName,$Params)=@_;
	local *TEMPLATE;
	my @Result;

	return undef unless open(TEMPLATE,$FileName);
	while (defined (my $Line=<TEMPLATE>)) {
		next if substr($Line,0,1) eq ';';
		chomp($Line);
		push(@Result,ExpandString($Line,$Params));
	}
	close(TEMPLATE);
	return \@Result;
}

sub PrintTemplate {
	my ($FileRef,$Template,$Params)=@_;

	foreach my $Line (@$Template) {
		next if substr($Line,0,1) eq ';';
		print $FileRef ExpandString($Line,$Params),"\n";
	}
}

sub PrintTemplateFromFile {
	my ($FileRef,$FileName,$Params)=@_;

	return undef unless open(TEMPLATE,$FileName);
	while (defined(my $Line=<TEMPLATE>)) {
		next if substr($Line,0,1) eq ';';
		chomp($Line);
		print $FileRef ExpandString($Line,$Params),"\n";
	}
	close(TEMPLATE);
}

# File Mask...
sub IsFileMask {
	my $File=shift;
	foreach my $Mask (@_) {
		my $RealMask=$Mask;
		$RealMask=~s/\./\\\./g;
		$RealMask=~s/\?/\./g;
		$RealMask=~s/\*/\.\*/g;
		return 1 if $File=~/^$RealMask$/i;
	}
	return 0;
}

# File Mask...
sub IsFileMaskWithCase {
	my $File=shift;
	foreach my $Mask (@_) {
		my $RealMask=$Mask;
		$RealMask=~s/\./\\\./g;
		$RealMask=~s/\?/\./g;
		$RealMask=~s/\*/\.\*/g;
		return 1 if $File=~/^$RealMask$/;
	}
	return 0;
}
